<?php

namespace Drupal\contact_emails;

use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Utility\Token;
use Drupal\Core\Session\AccountProxy;
use Drupal\Core\Render\Renderer;
use Drupal\Component\Render\FormattableMarkup;
use Egulias\EmailValidator\EmailValidator;

/**
 * Class ContactEmailerServiceProvider.
 *
 * @package Drupal\contact_emails
 */
class ContactEmailerServiceProvider {

  /**
   * Drupal\Core\Mail\MailManagerInterface definition.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * Drupal\Core\Utility\Token definition.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * Drupal\contact_emails\ContactEmails definition.
   *
   * @var \Drupal\contact_emails\ContactEmails
   */
  protected $contactEmails;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * Drupal\Core\Render\Renderer definition.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Egulias\EmailValidator\EmailValidator definition.
   *
   * @var \Egulias\EmailValidator\EmailValidator;
   */

  /**
   * Contact form machine name.
   *
   * @var string
   */
  protected $contactForm;

  /**
   * Contact message entity.
   *
   * @var object
   */
  protected $contactMessage;

  /**
   * Constructor.
   */
  public function __construct(
    MailManagerInterface $plugin_manager_mail,
    Token $token,
    ContactEmails $contact_emails,
    AccountProxy $current_user,
    Renderer $renderer,
    EmailValidator $email_validator
  ) {
    $this->mailManager = $plugin_manager_mail;
    $this->token = $token;
    $this->contactEmails = $contact_emails;
    $this->currentUser = $current_user;
    $this->renderer = $renderer;
    $this->emailValidator = $email_validator;
  }

  /**
   * Set the contact form.
   *
   * @param object $contactMessage
   *   The contact message.
   */
  public function setContactMessage($contactMessage) {
    $contactForm_object = $contactMessage->getContactForm();
    $this->contactForm = $contactForm_object->id();
    $this->contactMessage = $contactMessage;
  }

  /**
   * Send the emails.
   */
  public function sendEmails() {
    if ($emails = $this->getEmails()) {
      foreach ($emails as $email) {
        $module = 'contact_emails';
        $key = 'contact_emails';
        $to = $this->getTo($email);

        // Stop here if we don't know who to send to.
        if (!$to) {
          $error = $this->t('Unable to determine who to send the message to for for email id @id', [
            '@id' => $email->id,
          ]);
          drupal_set_message($error, 'warning', FALSE);
          continue;
        }

        // Subject and body.
        $params['subject'] = $email->subject;
        $params['message'] = $this->setBody($email->body);

        // Set to html mail by default.
        $params['format'] = 'text/html';

        // Final prep and send.
        $langcode = $this->currentUser->getPreferredLangcode();
        $send = TRUE;
        $reply = NULL;
        $result = $this->mailManager->mail($module, $key, $to, $langcode, $params, $reply, $send);
      }
    }
  }

  /**
   * Get who to send the email to.
   *
   * @param object $email
   *   The email settings.
   *
   * @return string
   *   The to string to be used by the mail manager.
   */
  protected function getTo($email) {
    $to = [];

    switch ($email->recipient_type) {
      case 'submitter':
        // Send to the submitter of the form.
        $to[] = $this->contactMessage->getSenderMail();
        break;

      case 'field':
        // Send to the value of an email field.
        if ($this->contactMessage->hasField($email->recipient_field)) {
          // Email could potentially be a repeating field.
          $emails = $this->contactMessage->get($email->recipient_field)->getValue();
          if ($emails) {
            foreach ($emails as $email) {
              if ($email['value']) {
                $to[] = $email['value'];
              }
            }
          }
        }
        break;

      case 'manual':
      default:
        // Send to the emails added via the manage email settings form. This is
        // already an array.
        $to = $email->recipients;
        break;
    }

    $to = $this->removeInvalidEmails($to);
    return implode(', ', $to);
  }

  /**
   * Remove invalid emails.
   *
   * @param array $emails
   *   An array of potentially valid emails.
   *
   * @return array
   *   An array of valid emails.
   */
  protected function removeInvalidEmails($emails) {
    $valid_emails = [];
    foreach ($emails as $email) {
      if ($this->emailValidator->isValid($email)) {
        $valid_emails[] = $email;
      }
      else {
        $error = $this->t('The following email does not appear to be valid and was not sent to: @email', [
          '@email' => $email,
        ]);
        drupal_set_message($error, 'warning', FALSE);
      }
    }
    return $valid_emails;
  }

  /**
   * Set the email message body.
   *
   * @var mixed $body
   *   A string or text format array.
   *
   * @return string
   *   The rendered html or plain text.
   */
  protected function setBody($body) {
    if (!$body) {
      return '';
    }

    $body = unserialize($body);

    // Render based on text format.
    if (!is_array($body)) {

      // No text format, plain text.
      $body = [
        '#plain_text' => $this->tokenizeBody($body),
      ];
    }
    else {

      // Has a text format, process the text.
      $body = [
        '#type' => 'processed_text',
        '#format' => $body['format'],
        '#text' => $this->tokenizeBody($body['value']),
      ];
    }

    // Render the body.
    $rendered_body = $this->renderer->render($body);

    // Send FormattableMarkup to ensure SwiftMailer doesn't further escape it.
    return new FormattableMarkup($rendered_body, []);
  }

  /**
   * Apply tokens to body value.
   *
   * @param string $body
   *   The body string value.
   *
   * @return string
   *   The tokenized body.
   */
  protected function tokenizeBody($body) {
    return $this->token->replace($body, [
      'contact_message' => $this->contactMessage,
    ]);
  }

  /**
   * Get the emails.
   */
  protected function getEmails() {
    $include_disabled = FALSE;
    return $this->contactEmails->getEmailsByContactForm($this->contactForm, $include_disabled);
  }

}
